@@ -94,13 +94,11 @@ namespace QuantLib {
       public:
         enum TimingAdjustment { Black76, BivariateLognormal };
         BlackIborCouponPricer(
-            const Handle<OptionletVolatilityStructure>& v =
-                Handle<OptionletVolatilityStructure>(),
+            const Handle<OptionletVolatilityStructure>& v = Handle<OptionletVolatilityStructure>(),
             const TimingAdjustment timingAdjustment = Black76,
-            const Handle<Quote> correlation =
-                Handle<Quote>(boost::shared_ptr<Quote>(new SimpleQuote(1.0))))
-            : IborCouponPricer(v), timingAdjustment_(timingAdjustment),
-              correlation_(correlation) {
+            const Handle<Quote>& correlation =
+                Handle<Quote>(ext::shared_ptr<Quote>(new SimpleQuote(1.0))))
+        : IborCouponPricer(v), timingAdjustment_(timingAdjustment), correlation_(correlation) {
             QL_REQUIRE(timingAdjustment_ == Black76 ||
                            timingAdjustment_ == BivariateLognormal,
                        "unknown timing adjustment (code " << timingAdjustment_
@@ -124,7 +122,7 @@ namespace QuantLib {
         Real gearing_;
         Spread spread_;
         Time accrualPeriod_;
-        boost::shared_ptr<IborIndex> index_;
+        ext::shared_ptr<IborIndex> index_;
         Real discount_;
         Real spreadLegValue_;
 
@@ -170,30 +168,30 @@ namespace QuantLib {
     };
 
     void setCouponPricer(const Leg& leg,
-                         const boost::shared_ptr<FloatingRateCouponPricer>&);
+                         const ext::shared_ptr<FloatingRateCouponPricer>&);
 
     void setCouponPricers(
             const Leg& leg,
-            const std::vector<boost::shared_ptr<FloatingRateCouponPricer> >&);
+            const std::vector<ext::shared_ptr<FloatingRateCouponPricer> >&);
 
     /*! set the first matching pricer (if any) to each coupon of the leg */
     void setCouponPricers(
             const Leg& leg,
-            const boost::shared_ptr<FloatingRateCouponPricer>&,
-            const boost::shared_ptr<FloatingRateCouponPricer>&);
+            const ext::shared_ptr<FloatingRateCouponPricer>&,
+            const ext::shared_ptr<FloatingRateCouponPricer>&);
 
     void setCouponPricers(
             const Leg& leg,
-            const boost::shared_ptr<FloatingRateCouponPricer>&,
-            const boost::shared_ptr<FloatingRateCouponPricer>&,
-            const boost::shared_ptr<FloatingRateCouponPricer>&);
+            const ext::shared_ptr<FloatingRateCouponPricer>&,
+            const ext::shared_ptr<FloatingRateCouponPricer>&,
+            const ext::shared_ptr<FloatingRateCouponPricer>&);
 
     void setCouponPricers(
             const Leg& leg,
-            const boost::shared_ptr<FloatingRateCouponPricer>&,
-            const boost::shared_ptr<FloatingRateCouponPricer>&,
-            const boost::shared_ptr<FloatingRateCouponPricer>&,
-            const boost::shared_ptr<FloatingRateCouponPricer>&);
+            const ext::shared_ptr<FloatingRateCouponPricer>&,
+            const ext::shared_ptr<FloatingRateCouponPricer>&,
+            const ext::shared_ptr<FloatingRateCouponPricer>&,
+            const ext::shared_ptr<FloatingRateCouponPricer>&);
 
     // inline
 
