@@ -17,6 +17,7 @@
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the license for more details.
 */
+
 #ifndef quantlib_randomdefault_latent_model_hpp
 #define quantlib_randomdefault_latent_model_hpp
 
@@ -25,6 +26,7 @@
 #include <ql/math/statistics/riskstatistics.hpp>
 #include <ql/math/solvers1d/brent.hpp>
 #include <ql/math/randomnumbers/sobolrsg.hpp>
+#include <ql/math/functional.hpp>
 #include <ql/experimental/credit/basket.hpp>
 #include <ql/experimental/credit/defaultlossmodel.hpp>
 
@@ -119,13 +121,13 @@ namespace QuantLib {
         void performCalculations() const {
             static_cast<const derivedRandomLM<copulaPolicy, USNG>* >(
                 this)->initDates();//in update?
-            copulasRng_ = boost::make_shared<copulaRNG_type>(copula_, seed_);
+            copulasRng_ = ext::make_shared<copulaRNG_type>(copula_, seed_);
             performSimulations();
         }
 
         void performSimulations() const {
             // Next sequence should determine the event and push it into buffer
-            for(Size i=nSims_; i; i--) {
+            for (Size i = nSims_; i != 0u; i--) {
                 const std::vector<Real>& sample =
                     copulasRng_->nextSequence().value;
                 static_cast<const derivedRandomLM<copulaPolicy, USNG>* >(
@@ -219,7 +221,7 @@ namespace QuantLib {
             USNG > > > > simsBuffer_;
 
         mutable copulaPolicy copula_;
-        mutable boost::shared_ptr<copulaRNG_type> copulasRng_;
+        mutable ext::shared_ptr<copulaRNG_type> copulasRng_;
 
         // Maximum time inversion horizon
         static const Size maxHorizon_ = 4050; // over 11 years
@@ -293,7 +295,8 @@ namespace QuantLib {
             }
         }
         std::transform(hitsByDate.begin(), hitsByDate.end(),
-            hitsByDate.begin(), std::bind2nd(std::divides<Real>(), Real(nSims_)));
+                       hitsByDate.begin(),
+                       divide_by<Real>(Real(nSims_)));
         return hitsByDate;
         // \todo Provide confidence interval
     }
@@ -509,7 +512,7 @@ namespace QuantLib {
         /*
         std::vector<Real>::iterator itPastPerc =
             std::find_if(losses.begin() + position, losses.end(),
-                std::bind1st(std::less<Real>(), perctlInf));
+                         greater_or_equal_to<Real>(perctlInf));
         // notice if the sample is flat at the end this might be zero
         Size pointsOverVal = nSims_ - std::distance(itPastPerc, losses.end());
         return pointsOverVal == 0 ? 0. :
@@ -633,7 +636,7 @@ namespace QuantLib {
         std::vector<Real> varLevels = splitVaRAndError(date, loss, 0.95)[0];
         // turn relative units into absolute:
         std::transform(varLevels.begin(), varLevels.end(), varLevels.begin(),
-            std::bind1st(std::multiplies<Real>(), loss));
+                       multiply_by<Real>(loss));
         return varLevels;
     }
 
@@ -752,7 +755,7 @@ namespace QuantLib {
     context of default
     See default transition models for another instance of this inversion.
     Alternatively use the faster trick (flat HR) mentioned in the code or make
-    the algorithm parametric on the type of interpolation in the DefautlTS
+    the algorithm parametric on the type of interpolation in the default TS.
     */
     namespace detail {// not template dependent .....move it
         //! Utility for the numerical time solver
@@ -760,8 +763,8 @@ namespace QuantLib {
           public:
             /* See a faster algorithm (neeeds to locate the points) in
             D.O'KANE p.249 sect 13.5 */
-            Root(const Handle<DefaultProbabilityTermStructure> dts, Real pd)
-                : dts_(dts), pd_(pd), curveRef_(dts->referenceDate()) {}
+            Root(const Handle<DefaultProbabilityTermStructure>& dts, Real pd)
+            : dts_(dts), pd_(pd), curveRef_(dts->referenceDate()) {}
             /* The cast I am forcing here comes from the requirement of 1D
             solvers to take in a target (cost) function of Real domain. It could
             be possible to change the template arg F in the 1D solvers to a
@@ -819,36 +822,31 @@ namespace QuantLib {
         typedef simEvent<RandomDefaultLM> defaultSimEvent;
 
         // \todo Consider this to be only a ConstantLossLM instead
-        const boost::shared_ptr<DefaultLatentModel<copulaPolicy> > model_;
+        const ext::shared_ptr<DefaultLatentModel<copulaPolicy> > model_;
         const std::vector<Real> recoveries_;
         // for time inversion:
         Real accuracy_;
     public:
         // \todo: Allow a constructor building its own default latent model.
-        RandomDefaultLM(
-            const boost::shared_ptr<DefaultLatentModel<copulaPolicy> >& model,
-            const std::vector<Real>& recoveries = std::vector<Real>(),
-            Size nSims = 0,// stats will crash on div by zero, FIX ME.
-            Real accuracy = 1.e-6,
-            BigNatural seed = 2863311530)
-        : RandomLM< ::QuantLib::RandomDefaultLM, copulaPolicy, USNG>
-            (model->numFactors(), model->size(), model->copula(),
-                nSims, seed ),
-          model_(model),
-          recoveries_(recoveries.size()==0 ? std::vector<Real>(model->size(),
-            0.) : recoveries),
-          accuracy_(accuracy)
-        {
-            // redundant through basket?
-            this->registerWith(Settings::instance().evaluationDate());
-            this->registerWith(model_);
+      explicit RandomDefaultLM(const ext::shared_ptr<DefaultLatentModel<copulaPolicy> >& model,
+                               const std::vector<Real>& recoveries = std::vector<Real>(),
+                               Size nSims = 0, // stats will crash on div by zero, FIX ME.
+                               Real accuracy = 1.e-6,
+                               BigNatural seed = 2863311530UL)
+      : RandomLM< ::QuantLib::RandomDefaultLM, copulaPolicy, USNG>(
+            model->numFactors(), model->size(), model->copula(), nSims, seed),
+        model_(model),
+        recoveries_(recoveries.empty() ? std::vector<Real>(model->size(), 0.) : recoveries),
+        accuracy_(accuracy) {
+          // redundant through basket?
+          this->registerWith(Settings::instance().evaluationDate());
+          this->registerWith(model_);
         }
-        RandomDefaultLM(
-            const boost::shared_ptr<ConstantLossLatentmodel<copulaPolicy> >&
-                model,
+        explicit RandomDefaultLM(
+            const ext::shared_ptr<ConstantLossLatentmodel<copulaPolicy> >& model,
             Size nSims = 0,// stats will crash on div by zero, FIX ME.
             Real accuracy = 1.e-6,
-            BigNatural seed = 2863311530)
+            BigNatural seed = 2863311530UL)
         : RandomLM< ::QuantLib::RandomDefaultLM, copulaPolicy, USNG>
             (model->numFactors(), model->size(), model->copula(),
                 nSims, seed ),
@@ -883,7 +881,7 @@ namespace QuantLib {
             Date today = Settings::instance().evaluationDate();
             Date maxHorizonDate = today  + Period(this->maxHorizon_, Days);
 
-            const boost::shared_ptr<Pool>& pool = this->basket_->pool();
+            const ext::shared_ptr<Pool>& pool = this->basket_->pool();
             for(Size iName=0; iName < this->basket_->size(); ++iName)//use'live'
                 horizonDefaultPs_.push_back(pool->get(pool->names()[iName]).
                     defaultProbability(this->basket_->defaultKeys()[iName])
@@ -935,7 +933,7 @@ namespace QuantLib {
     void RandomDefaultLM<C, URNG>::nextSample(
         const std::vector<Real>& values) const
     {
-        const boost::shared_ptr<Pool>& pool = this->basket_->pool();
+        const ext::shared_ptr<Pool>& pool = this->basket_->pool();
         // starts with no events
         this->simsBuffer_.push_back(std::vector<defaultSimEvent> ());
 
