@@ -39,13 +39,6 @@ FOR A PARTICULAR PURPOSE.  See the license for more details.
 
 #ifndef QL_ENABLE_THREAD_SAFE_OBSERVER_PATTERN
 
-// Boost libraries prior to 1.47 have a bug in the hash function,
-// which makes boost::unordered_set very inefficient if the key is of type
-// ext::shared_ptr. In this case fall back to std::set.
-#if BOOST_VERSION < 104700
-#include <set>
-#endif
-
 namespace QuantLib {
 
     class Observer;
@@ -106,11 +99,7 @@ namespace QuantLib {
     /*! \ingroup patterns */
     class Observer {
       public:
-#if BOOST_VERSION < 104700
-        typedef std::set<ext::shared_ptr<Observable> > set_type;
-#else
         typedef boost::unordered_set<ext::shared_ptr<Observable> > set_type;
-#endif
         typedef set_type::iterator iterator;
 
         // constructors, assignment, destructor
@@ -219,7 +208,7 @@ namespace QuantLib {
 
     inline std::pair<Observer::iterator, bool>
     Observer::registerWith(const ext::shared_ptr<Observable>& h) {
-        if (h) {
+        if (h != 0) {
             h->registerObserver(this);
             return observables_.insert(h);
         }
@@ -228,7 +217,7 @@ namespace QuantLib {
 
     inline void
     Observer::registerWithObservables(const ext::shared_ptr<Observer> &o) {
-        if (o) {
+        if (o != 0) {
             iterator i;
             for (i = o->observables_.begin(); i != o->observables_.end(); ++i)
                 registerWith(*i);
@@ -237,7 +226,7 @@ namespace QuantLib {
 
     inline
     Size Observer::unregisterWith(const ext::shared_ptr<Observable>& h) {
-        if (h)
+        if (h != 0)
             h->unregisterObserver(this);
         return observables_.erase(h);
     }
@@ -261,17 +250,17 @@ namespace QuantLib {
 #include <boost/thread/mutex.hpp>
 #include <boost/thread/recursive_mutex.hpp>
 #include <boost/smart_ptr/owner_less.hpp>
-#include <boost/enable_shared_from_this.hpp>
 #include <set>
 
-namespace QuantLib {
 
+
+namespace QuantLib {
     class Observable;
     class ObservableSettings;
 
     //! Object that gets notified when a given observable changes
     /*! \ingroup patterns */
-    class Observer : public boost::enable_shared_from_this<Observer> {
+    class Observer : public ext::enable_shared_from_this<Observer> {
         friend class Observable;
         friend class ObservableSettings;
       public:
@@ -318,9 +307,14 @@ namespace QuantLib {
             void update() const {
                 boost::lock_guard<boost::recursive_mutex> lock(mutex_);
                 if (active_) {
+                    // c++17 is required if used with std::shared_ptr<T>
                     const ext::weak_ptr<Observer> o
                         = observer_->weak_from_this();
-                    if (!o._empty()) {
+
+                    //check for empty weak reference
+                    //https://stackoverflow.com/questions/45507041/how-to-check-if-weak-ptr-is-empty-non-assigned
+                    const ext::weak_ptr<Observer> empty;
+                    if (o.owner_before(empty) || empty.owner_before(o)) {
                         const ext::shared_ptr<Observer> obs(o.lock());
                         if (obs)
                             obs->update();
