@@ -31,16 +31,10 @@
 /*! @{ */
 
 //! version string
-#ifdef QL_DEBUG
-    #define QL_VERSION "1.18-debug"
-#else
-    #define QL_VERSION "1.18"
-#endif
+#define QL_VERSION "1.19"
 
 //! version hexadecimal number
-#define QL_HEX_VERSION 0x011800f0
-//! version string for output lib name
-#define QL_LIB_VERSION "1_18"
+#define QL_HEX_VERSION 0x011900f0
 
 /*! @}  */
 
