@@ -208,13 +208,13 @@ void MatricesTest::testSVD() {
         Matrix& A = testMatrices[j];
         SVD svd(A);
         // U is m x n
-        Matrix U = svd.U();
+        const Matrix& U = svd.U();
         // s is n long
         Array s = svd.singularValues();
         // S is n x n
         Matrix S = svd.S();
         // V is n x n
-        Matrix V = svd.V();
+        const Matrix& V = svd.V();
 
         for (Size i=0; i < S.rows(); i++) {
             if (S[i][i] != s[i])
@@ -410,7 +410,7 @@ void MatricesTest::testDeterminant() {
         for (Matrix::iterator iter = m.begin(); iter != m.end(); ++iter)
             *iter = rng.next().value;
 
-        if (!(j%3)) {
+        if ((j % 3) == 0u) {
             // every third matrix is a singular matrix
             Size row = Size(3*rng.next().value);
             std::fill(m.row_begin(row), m.row_end(row), 0.0);
